@@ -4,7 +4,7 @@ This module holds the code of the Markdown extension responsible for matching "a
 The extension is composed of a Markdown [block processor](https://python-markdown.github.io/extensions/api/#blockparser)
 that matches indented blocks starting with a line like '::: identifier'.
 
-For each of these blocks, it uses a [handler][mkdocstrings.handlers.BaseHandler] to collect documentation about
+For each of these blocks, it uses a [handler][mkdocstrings.handlers.base.BaseHandler] to collect documentation about
 the given identifier and render it with Jinja templates.
 
 Both the collection and rendering process can be configured by adding YAML configuration under the "autodoc"
@@ -22,9 +22,9 @@ instruction:
       option_x: etc
 ```
 """
-import logging
 import re
-from typing import Any, Tuple
+from collections import ChainMap
+from typing import Any, Mapping, MutableSequence, Tuple
 from xml.etree.ElementTree import XML, Element, ParseError  # noqa: S405 (we choose to trust the XML input)
 
 import yaml
@@ -34,25 +34,26 @@ from markdown.blockparser import BlockParser
 from markdown.blockprocessors import BlockProcessor
 from markdown.extensions import Extension
 from markdown.util import AtomicString
-from mkdocs.utils import warning_filter
 
-from mkdocstrings.handlers import CollectionError, get_handler
+from mkdocstrings.handlers.base import CollectionError, Handlers
+from mkdocstrings.loggers import get_logger
+from mkdocstrings.references import AutoRefInlineProcessor
 
-log = logging.getLogger(f"mkdocs.plugins.{__name__}")
-log.addFilter(warning_filter)
+log = get_logger(__name__)
 
 ENTITIES = """
     <!DOCTYPE html [
-        <!ENTITY nbsp '&amp;nbsp;'>
-        <!ENTITY lsquo '&amp;lsquo;'>
-        <!ENTITY rsquo '&amp;rsquo;'>
-        <!ENTITY ldquo '&amp;ldquo;'>
-        <!ENTITY rdquo '&amp;rdquo;'>
-        <!ENTITY laquo '&amp;laquo;'>
-        <!ENTITY raquo '&amp;raquo;'>
         <!ENTITY hellip '&amp;hellip;'>
-        <!ENTITY ndash '&amp;ndash;'>
+        <!ENTITY laquo '&amp;laquo;'>
+        <!ENTITY ldquo '&amp;ldquo;'>
+        <!ENTITY lsquo '&amp;lsquo;'>
         <!ENTITY mdash '&amp;mdash;'>
+        <!ENTITY nbsp '&amp;nbsp;'>
+        <!ENTITY ndash '&amp;ndash;'>
+        <!ENTITY para '&amp;para;'>
+        <!ENTITY raquo '&amp;raquo;'>
+        <!ENTITY rdquo '&amp;rdquo;'>
+        <!ENTITY rsquo '&amp;rsquo;'>
     ]>
 """
 
@@ -92,10 +93,9 @@ class AutoDocProcessor(BlockProcessor):
     a matched block.
     """
 
-    classname = "autodoc"
-    regex = re.compile(r"(?:^|\n)::: ?([:a-zA-Z0-9_.-]*) *(?:\n|$)")
+    regex = re.compile(r"^(?P<heading>#{1,6} *|)::: ?(?P<name>.+?) *$", flags=re.MULTILINE)
 
-    def __init__(self, parser: BlockParser, md: Markdown, config: dict) -> None:
+    def __init__(self, parser: BlockParser, md: Markdown, config: dict, handlers: Handlers) -> None:
         """
         Initialize the object.
 
@@ -104,32 +104,28 @@ class AutoDocProcessor(BlockProcessor):
             md: A `markdown.Markdown` instance.
             config: The [configuration][mkdocstrings.plugin.MkdocstringsPlugin.config_scheme]
                 of the `mkdocstrings` plugin.
+            handlers: A [mkdocstrings.handlers.base.Handlers][] instance.
         """
         super().__init__(parser=parser)
         self.md = md
         self._config = config
+        self._handlers = handlers
+        self._updated_env = False
 
-    def test(self, parent: Element, block: Element) -> bool:
+    def test(self, parent: Element, block: str) -> bool:
         """
         Match our autodoc instructions.
 
         Arguments:
-            parent:parent: The parent element in the XML tree.
+            parent: The parent element in the XML tree.
             block: The block to be tested.
 
         Returns:
             Whether this block should be processed or not.
         """
-        sibling = self.lastChild(parent)
-        bool1 = self.regex.search(str(block))
-        bool2 = (
-            str(block).startswith(" " * self.tab_length)
-            and sibling is not None
-            and sibling.get("class", "").find(self.classname) != -1
-        )
-        return bool(bool1 or bool2)
+        return bool(self.regex.search(block))
 
-    def run(self, parent: Element, blocks: Element) -> None:
+    def run(self, parent: Element, blocks: MutableSequence[str]) -> None:
         """
         Run code on the matched blocks.
 
@@ -141,18 +137,21 @@ class AutoDocProcessor(BlockProcessor):
             blocks: The rest of the blocks to be processed.
         """
         block = blocks.pop(0)
-        match = self.regex.search(str(block))
+        match = self.regex.search(block)
 
         if match:
+            if match.start() > 0:
+                self.parser.parseBlocks(parent, [block[: match.start()]])
             # removes the first line
             block = block[match.end() :]  # type: ignore
 
         block, the_rest = self.detab(block)
 
         if match:
-            identifier = match.group(1)
-            log.debug(f"mkdocstrings.extension: Matched '::: {identifier}'")
-            xml_element = self.process_block(identifier, str(block))
+            identifier = match["name"]
+            heading_level = match["heading"].count("#")
+            log.debug(f"Matched '::: {identifier}'")
+            xml_element = self.process_block(identifier, block, heading_level)
             parent.append(xml_element)
 
         if the_rest:
@@ -161,95 +160,68 @@ class AutoDocProcessor(BlockProcessor):
             # list for future processing.
             blocks.insert(0, the_rest)
 
-    def process_block(self, identifier: str, yaml_block: str) -> Element:
+    def process_block(self, identifier: str, yaml_block: str, heading_level: int = 0) -> Element:
         """
         Process an autodoc block.
 
         Arguments:
             identifier: The identifier of the object to collect and render.
             yaml_block: The YAML configuration.
+            heading_level: Suggested level of the the heading to insert (0 to ignore).
 
         Raises:
             CollectionError: When something wrong happened during collection.
+            ParseError: When the generated HTML could not be parsed as XML.
+            TemplateNotFound: When a template used for rendering could not be found.
 
         Returns:
             A new XML element.
         """
         config = yaml.safe_load(yaml_block) or {}
-        handler_name = self.get_handler_name(config)
+        handler_name = self._handlers.get_handler_name(config)
 
-        log.debug(f"mkdocstrings.extension: Using handler '{handler_name}'")
-        handler_config = self.get_handler_config(handler_name)
-        handler = get_handler(
-            handler_name,
-            self._config["theme_name"],
-            self._config["mkdocstrings"]["custom_templates"],
-            **handler_config,
-        )
+        log.debug(f"Using handler '{handler_name}'")
+        handler_config = self._handlers.get_handler_config(handler_name)
+        handler = self._handlers.get_handler(handler_name, handler_config)
 
         selection, rendering = get_item_configs(handler_config, config)
+        if heading_level:
+            rendering = ChainMap(rendering, {"heading_level": heading_level})  # like setdefault
 
-        log.debug("mkdocstrings.extension: Collecting data")
+        log.debug("Collecting data")
         try:
             data: Any = handler.collector.collect(identifier, selection)
         except CollectionError:
-            log.error(f"mkdocstrings.extension: Could not collect '{identifier}'")
+            log.error(f"Could not collect '{identifier}'")
             raise
 
-        log.debug("mkdocstrings.extension: Updating renderer's env")
-        handler.renderer.update_env(self.md, self._config)
+        if not self._updated_env:
+            log.debug("Updating renderer's env")
+            handler.renderer.update_env(self.md, self._config)
+            self._updated_env = True
 
-        log.debug("mkdocstrings.extension: Rendering templates")
+        log.debug("Rendering templates")
         try:
             rendered = handler.renderer.render(data, rendering)
         except TemplateNotFound as exc:
             theme_name = self._config["theme_name"]
             log.error(
-                f"mkdocstrings.extension: Template '{exc.name}' not found "
-                f"for '{handler_name}' handler and theme '{theme_name}'.",
+                f"Template '{exc.name}' not found for '{handler_name}' handler and theme '{theme_name}'.",
             )
             raise
 
-        log.debug("mkdocstrings.extension: Loading HTML back into XML tree")
+        log.debug("Loading HTML back into XML tree")
+        rendered = ENTITIES + rendered
         try:
-            xml_contents = XML(ENTITIES + rendered)
+            xml_contents = XML(rendered)
         except ParseError as error:
             log_xml_parse_error(str(error), rendered)
             raise
 
         return atomic_brute_cast(xml_contents)  # type: ignore
 
-    def get_handler_name(self, config: dict) -> str:
-        """
-        Return the handler name defined in an "autodoc" instruction YAML configuration, or the global default handler.
 
-        Arguments:
-            config: A configuration dictionary, obtained from YAML below the "autodoc" instruction.
-
-        Returns:
-            The name of the handler to use.
-        """
-        if "handler" in config:
-            return config["handler"]
-        return self._config["mkdocstrings"]["default_handler"]
-
-    def get_handler_config(self, handler_name: str) -> dict:
-        """
-        Return the global configuration of the given handler.
-
-        Arguments:
-            handler_name: The name of the handler to get the global configuration of.
-
-        Returns:
-            The global configuration of the given handler. It can be an empty dictionary.
-        """
-        handlers = self._config["mkdocstrings"].get("handlers", {})
-        if handlers:
-            return handlers.get(handler_name, {})
-        return {}
-
-
-def get_item_configs(handler_config: dict, config: dict) -> Tuple[dict, dict]:
+def get_item_configs(handler_config: dict, config: dict) -> Tuple[Mapping, Mapping]:
     """
     Get the selection and rendering configuration merged into the global configuration of the given handler.
 
@@ -260,10 +232,8 @@ def get_item_configs(handler_config: dict, config: dict) -> Tuple[dict, dict]:
     Returns:
         Two dictionaries: selection and rendering. The local configurations are merged into the global ones.
     """
-    item_selection_config = dict(handler_config.get("selection", {}))
-    item_selection_config.update(config.get("selection", {}))
-    item_rendering_config = dict(handler_config.get("rendering", {}))
-    item_rendering_config.update(config.get("rendering", {}))
+    item_selection_config = ChainMap(config.get("selection", {}), handler_config.get("selection", {}))
+    item_rendering_config = ChainMap(config.get("rendering", {}), handler_config.get("rendering", {}))
     return item_selection_config, item_rendering_config
 
 
@@ -277,20 +247,26 @@ def log_xml_parse_error(error: str, xml_text: str) -> None:
         error: The error message (no traceback).
         xml_text: The XML text that generated the parsing error.
     """
-    message = f"mkdocstrings.extension: {error}"
-    if "mismatched tag" in error:
+    message = error
+    mismatched_tag = "mismatched tag" in error
+    undefined_entity = "undefined entity" in error
+
+    if mismatched_tag or undefined_entity:
         line_column = error[error.rfind(":") + 1 :]
         line, column = line_column.split(", ")
         lineno = int(line[line.rfind(" ") + 1 :])
         columnno = int(column[column.rfind(" ") + 1 :])
 
         line = xml_text.split("\n")[lineno - 1]
-        character = line[columnno]
-        message += (
-            f" (character {character}):\n{line}\n"
-            "If your Markdown contains angle brackets < >, try to wrap them between backticks `< >`, "
-            "or replace them with &lt; and &gt;"
-        )
+        if mismatched_tag:
+            character = line[columnno]
+            message += (
+                f" (character {character}):\n{line}\n"
+                "If your Markdown contains angle brackets < >, try to wrap them between backticks `< >`, "
+                "or replace them with &lt; and &gt;"
+            )
+        elif undefined_entity:
+            message += f":\n{line}\n"
     log.error(message)
 
 
@@ -301,19 +277,22 @@ class MkdocstringsExtension(Extension):
     It cannot work outside of `mkdocstrings`.
     """
 
-    priority = 110
+    blockprocessor_priority = 75  # Right before markdown.blockprocessors.HashHeaderProcessor
+    inlineprocessor_priority = 168  # Right after markdown.inlinepatterns.ReferenceInlineProcessor
 
-    def __init__(self, config: dict, **kwargs) -> None:
+    def __init__(self, config: dict, handlers: Handlers, **kwargs) -> None:
         """
         Initialize the object.
 
         Arguments:
             config: The configuration items from `mkdocs` and `mkdocstrings` that must be passed to the block processor
                 when instantiated in [`extendMarkdown`][mkdocstrings.extension.MkdocstringsExtension.extendMarkdown].
+            handlers: A [mkdocstrings.handlers.base.Handlers][] instance.
             kwargs: Keyword arguments used by `markdown.extensions.Extension`.
         """
         super().__init__(**kwargs)
         self._config = config
+        self._handlers = handlers
 
     def extendMarkdown(self, md: Markdown) -> None:  # noqa: N802 (casing: parent method's name)
         """
@@ -325,5 +304,7 @@ class MkdocstringsExtension(Extension):
             md: A `markdown.Markdown` instance.
         """
         md.registerExtension(self)
-        processor = AutoDocProcessor(md.parser, md, self._config)
-        md.parser.blockprocessors.register(processor, "mkdocstrings", self.priority)
+        processor = AutoDocProcessor(md.parser, md, self._config, self._handlers)
+        md.parser.blockprocessors.register(processor, "mkdocstrings", self.blockprocessor_priority)
+        ref_processor = AutoRefInlineProcessor(md)
+        md.inlinePatterns.register(ref_processor, "mkdocstrings", self.inlineprocessor_priority)
