@@ -5,19 +5,18 @@ The handler collects data with [`pytkdocs`](https://github.com/pawamoy/pytkdocs)
 """
 
 import json
-import logging
 import os
 import sys
+from collections import ChainMap
 from subprocess import PIPE, Popen  # noqa: S404 (what other option, more secure that PIPE do we have? sockets?)
 from typing import Any, List, Optional
 
 from markdown import Markdown
-from mkdocs.utils import warning_filter
 
-from mkdocstrings.handlers import BaseCollector, BaseHandler, BaseRenderer, CollectionError
+from mkdocstrings.handlers.base import BaseCollector, BaseHandler, BaseRenderer, CollectionError
+from mkdocstrings.loggers import get_logger
 
-log = logging.getLogger(f"mkdocs.plugins.{__name__}")
-log.addFilter(warning_filter)
+log = get_logger(__name__)
 
 
 class PythonRenderer(BaseRenderer):
@@ -25,7 +24,7 @@ class PythonRenderer(BaseRenderer):
     The class responsible for loading Jinja templates and rendering them.
 
     It defines some configuration options, implements the `render` method,
-    and overrides the `update_env` method of the [`BaseRenderer` class][mkdocstrings.handlers.BaseRenderer].
+    and overrides the `update_env` method of the [`BaseRenderer` class][mkdocstrings.handlers.base.BaseRenderer].
 
     Attributes:
         fallback_theme: The theme to fallback to.
@@ -43,6 +42,7 @@ class PythonRenderer(BaseRenderer):
         "show_object_full_path": False,
         "show_category_heading": False,
         "show_if_no_docstring": False,
+        "show_signature_annotations": False,
         "show_source": True,
         "group_by_category": True,
         "heading_level": 2,
@@ -59,21 +59,21 @@ class PythonRenderer(BaseRenderer):
     **`show_root_members_full_path`** | `bool` | Show the full Python path of objects that are children of the root object (for example, classes in a module). When False, `show_object_full_path` overrides. | `False`
     **`show_category_heading`** | `bool` | When grouped by categories, show a heading for each category. | `False`
     **`show_if_no_docstring`** | `bool` | Show the object heading even if it has no docstring or children with docstrings. | `False`
+    **`show_signature_annotations`** | `bool` | Show the type annotations in methods and functions signatures. | `False`
     **`show_source`** | `bool` | Show the source code of this object. | `True`
     **`group_by_category`** | `bool` | Group the object's children by categories: attributes, classes, functions, methods, and modules. | `True`
     **`heading_level`** | `int` | The initial heading level to use. | `2`
     """  # noqa: E501
 
     def render(self, data: Any, config: dict) -> str:  # noqa: D102 (ignore missing docstring)
-        final_config = dict(self.default_config)
-        final_config.update(config)
+        final_config = ChainMap(config, self.default_config)
 
         template = self.env.get_template(f"{data['category']}.html")
 
         # Heading level is a "state" variable, that will change at each step
         # of the rendering recursion. Therefore, it's easier to use it as a plain value
         # than as an item in a dictionary.
-        heading_level = final_config.pop("heading_level")
+        heading_level = final_config["heading_level"]
 
         return template.render(
             **{"config": final_config, data["category"]: data, "heading_level": heading_level, "root": True},
@@ -91,7 +91,7 @@ class PythonCollector(BaseCollector):
     The class responsible for loading Jinja templates and rendering them.
 
     It defines some configuration options, implements the `render` method,
-    and overrides the `update_env` method of the [`BaseRenderer` class][mkdocstrings.handlers.BaseRenderer].
+    and overrides the `update_env` method of the [`BaseRenderer` class][mkdocstrings.handlers.base.BaseRenderer].
     """
 
     default_config: dict = {"filters": ["!^_[^_]"]}
@@ -130,7 +130,7 @@ class PythonCollector(BaseCollector):
         Arguments:
             setup_commands: A list of python commands as strings to be executed in the subprocess before `pytkdocs`.
         """
-        log.debug("mkdocstrings.handlers.python: Opening 'pytkdocs' subprocess")
+        log.debug("Opening 'pytkdocs' subprocess")
         env = os.environ.copy()
         env["PYTHONUNBUFFERED"] = "1"
 
@@ -192,52 +192,51 @@ class PythonCollector(BaseCollector):
         Returns:
             The collected object-tree.
         """
-        final_config = dict(self.default_config)
-        final_config.update(config)
+        final_config = ChainMap(config, self.default_config)
 
-        log.debug("mkdocstrings.handlers.python: Preparing input")
+        log.debug("Preparing input")
         json_input = json.dumps({"objects": [{"path": identifier, **final_config}]})
 
-        log.debug("mkdocstrings.handlers.python: Writing to process' stdin")
+        log.debug("Writing to process' stdin")
         self.process.stdin.write(json_input + "\n")  # type: ignore
         self.process.stdin.flush()  # type: ignore
 
-        log.debug("mkdocstrings.handlers.python: Reading process' stdout")
+        log.debug("Reading process' stdout")
         stdout = self.process.stdout.readline()  # type: ignore
 
-        log.debug("mkdocstrings.handlers.python: Loading JSON output as Python object")
+        log.debug("Loading JSON output as Python object")
         try:
             result = json.loads(stdout)
         except json.decoder.JSONDecodeError as exception:
-            log.error(f"mkdocstrings.handlers.python: Error while loading JSON: {stdout}")
+            log.error(f"Error while loading JSON: {stdout}")
             raise CollectionError(str(exception)) from exception
 
         error = result.get("error")
         if error:
-            message = f"mkdocstrings.handlers.python: Collection failed: {error}"
+            message = f"Collection failed: {error}"
             if "traceback" in result:
                 message += f"\n{result['traceback']}"
             log.error(message)
             raise CollectionError(error)
 
         for loading_error in result["loading_errors"]:
-            log.warning(f"mkdocstrings.handlers.python: {loading_error}")
+            log.warning(loading_error)
 
         for errors in result["parsing_errors"].values():
             for parsing_error in errors:
-                log.warning(f"mkdocstrings.handlers.python: {parsing_error}")
+                log.warning(parsing_error)
 
         # We always collect only one object at a time
         result = result["objects"][0]
 
-        log.debug("mkdocstrings.handlers.python: Rebuilding categories and children lists")
+        log.debug("Rebuilding categories and children lists")
         rebuild_category_lists(result)
 
         return result
 
     def teardown(self) -> None:
         """Terminate the opened subprocess, set it to `None`."""
-        log.debug("mkdocstrings.handlers.python: Tearing process down")
+        log.debug("Tearing process down")
         self.process.terminate()
 
 
