@@ -15,8 +15,8 @@ this plugin searches for references of the form `[identifier][]` or `[title][ide
 and fixes them using the previously stored identifier-URL mapping.
 
 Once the documentation is built, the [`on_post_build` event hook](https://www.mkdocs.org/user-guide/plugins/#on_post_build)
-is triggered and calls the [`handlers.teardown()` method][mkdocstrings.handlers.teardown]. This method is used
-to teardown the [handlers][mkdocstrings.handlers] that were instantiated during documentation buildup.
+is triggered and calls the [`handlers.teardown()` method][mkdocstrings.handlers.base.Handlers.teardown]. This method is
+used to teardown the handlers that were instantiated during documentation buildup.
 
 Finally, when serving the documentation, it can add directories to watch
 during the [`on_serve` event hook](https://www.mkdocs.org/user-guide/plugins/#on_serve).
@@ -32,14 +32,13 @@ from mkdocs.config.config_options import Type as MkType
 from mkdocs.plugins import BasePlugin
 from mkdocs.structure.pages import Page
 from mkdocs.structure.toc import AnchorLink
-from mkdocs.utils import warning_filter
 
 from mkdocstrings.extension import MkdocstringsExtension
-from mkdocstrings.handlers import teardown
+from mkdocstrings.handlers.base import BaseHandler, Handlers
+from mkdocstrings.loggers import get_logger
 from mkdocstrings.references import fix_refs
 
-log = logging.getLogger(f"mkdocs.plugins.{__name__}")
-log.addFilter(warning_filter)
+log = get_logger(__name__)
 
 SELECTION_OPTS_KEY: str = "selection"
 """The name of the selection parameter in YAML configuration blocks."""
@@ -103,8 +102,8 @@ class MkdocstringsPlugin(BasePlugin):
     def __init__(self) -> None:
         """Initialize the object."""
         super().__init__()
-        self.mkdocstrings_extension: Optional[MkdocstringsExtension] = None
         self.url_map: Dict[Any, str] = {}
+        self.handlers: Optional[Handlers] = None
 
     def on_serve(self, server: Server, builder: Callable = None, **kwargs) -> Server:  # noqa: W0613 (unused arguments)
         """
@@ -126,9 +125,9 @@ class MkdocstringsPlugin(BasePlugin):
         if builder is None:
             # The builder parameter was added in mkdocs v1.1.1.
             # See issue https://github.com/mkdocs/mkdocs/issues/1952.
-            builder = list(server.watcher._tasks.values())[0]["func"]  # noqa: WPS437 (protected attribute)
+            builder = list(server.watcher._tasks.values())[0]["func"]  # noqa: W0212 (protected member)
         for element in self.config["watch"]:
-            log.debug(f"mkdocstrings.plugin: Adding directory '{element}' to watcher")
+            log.debug(f"Adding directory '{element}' to watcher")
             server.watch(element, builder)
         return server
 
@@ -150,7 +149,7 @@ class MkdocstringsPlugin(BasePlugin):
         Returns:
             The modified config.
         """
-        log.debug("mkdocstrings.plugin: Adding extension to the list")
+        log.debug("Adding extension to the list")
 
         theme_name = None
         if config["theme"].name is None:
@@ -165,8 +164,9 @@ class MkdocstringsPlugin(BasePlugin):
             "mkdocstrings": self.config,
         }
 
-        self.mkdocstrings_extension = MkdocstringsExtension(config=extension_config)
-        config["markdown_extensions"].append(self.mkdocstrings_extension)
+        self.handlers = Handlers(extension_config)
+        mkdocstrings_extension = MkdocstringsExtension(extension_config, self.handlers)
+        config["markdown_extensions"].append(mkdocstrings_extension)
         return config
 
     def on_page_content(self, html: str, page: Page, **kwargs) -> str:  # noqa: W0613 (unused arguments)
@@ -186,7 +186,7 @@ class MkdocstringsPlugin(BasePlugin):
         Returns:
             The same HTML. We only use this hook to map anchors to URLs.
         """
-        log.debug(f"mkdocstrings.plugin: Mapping identifiers to URLs for page {page.file.src_path}")
+        log.debug(f"Mapping identifiers to URLs for page {page.file.src_path}")
         for item in page.toc.items:
             self.map_urls(page.url, item)
         return html
@@ -216,7 +216,7 @@ class MkdocstringsPlugin(BasePlugin):
 
         We log a warning for each reference that we couldn't map to an URL, but try to be smart and ignore identifiers
         that do not look legitimate (sometimes documentation can contain strings matching
-        our [`AUTO_REF`][mkdocstrings.references.AUTO_REF] regular expression that did not intend to reference anything).
+        our [`AUTO_REF_RE`][mkdocstrings.references.AUTO_REF_RE] regular expression that did not intend to reference anything).
         We currently ignore references when their identifier contains a space or a slash.
 
         Arguments:
@@ -227,14 +227,14 @@ class MkdocstringsPlugin(BasePlugin):
         Returns:
             Modified HTML.
         """
-        log.debug(f"mkdocstrings.plugin: Fixing references in page {page.file.src_path}")
+        log.debug(f"Fixing references in page {page.file.src_path}")
 
         fixed_output, unmapped = fix_refs(output, page.url, self.url_map)
 
         if unmapped and log.isEnabledFor(logging.WARNING):
             for ref in unmapped:
                 log.warning(
-                    f"mkdocstrings.plugin: {page.file.src_path}: Could not find cross-reference target '[{ref}]'",
+                    f"{page.file.src_path}: Could not find cross-reference target '[{ref}]'",
                 )
 
         return fixed_output
@@ -255,5 +255,23 @@ class MkdocstringsPlugin(BasePlugin):
         Arguments:
             kwargs: Additional arguments passed by MkDocs.
         """
-        log.debug("mkdocstrings.plugin: Tearing handlers down")
-        teardown()
+        if self.handlers:
+            log.debug("Tearing handlers down")
+            self.handlers.teardown()
+
+    def get_handler(self, handler_name: str) -> BaseHandler:
+        """
+        Get a handler by its name. See [mkdocstrings.handlers.base.Handlers.get_handler][].
+
+        Arguments:
+            handler_name: The name of the handler.
+
+        Raises:
+            RuntimeError: If the plugin hasn't been initialized with a config.
+
+        Returns:
+            An instance of a subclass of [`BaseHandler`][mkdocstrings.handlers.base.BaseHandler].
+        """
+        if not self.handlers:
+            raise RuntimeError("The plugin hasn't been initialized with a config yet")
+        return self.handlers.get_handler(handler_name)
